 
{ Analog Devices, Inc 
  One Technology Way 
  Norwood, MA 02062-9106 
  DSP Applications Assistance Tel: 617-461-3672 
  Fax: 617-461-3010 
 
  September 1990 
 
  This code is intended to perform a basic functional test of an ADSP-2101. 
  The test is not exhaustive but does touch all sections of the part. A 
  reasonable level of confidence in the functionality of the processor can be 
  obtained by passing the test. Throughout this code, comments describe what  
  is being tested and to what standards.  Should enhancement become necessary, 
  it can be easily accomplished.  Unnecessary sections of the code can also be 
  removed freely, as each test is completely modular. If there are concerns  
  or comments, please call for DSP Applications Assistance at 617-461-3672. 
} 
 
.MODULE/RAM	functional_test; 
 
.VAR/DM/RAM	jump_test_result, 
		call_test_result, 
		loop_test_result, 
		dp_test_result, 
		ALU_func_result, 
		DAG1_result, 
		DAG2_result, 
		MAC_result, 
		SHIFTER_result, 
		TIMER_result; 
 
{------------------------------------------------------------------------------} 
{ Reset vector and Interrupt Table                                             } 
{------------------------------------------------------------------------------} 
 
reset_vector:	JUMP test_setup; NOP; NOP; NOP; 
irq2_vector:	RTI; NOP; NOP; NOP; 
trans0_vector:	RTI; NOP; NOP; NOP; 
recv0_vector:	RTI; NOP; NOP; NOP; 
trans1_vector:	RTI; NOP; NOP; NOP; 
recv1_vector:	RTI; NOP; NOP; NOP; 
timer_vector:	AX0 = 0xA1A1; RTI; NOP; NOP; 
 
{------------------------------------------------------------------------------} 
{ Setup data memory result variables for self test.  If tests are PASSED each  } 
{ test_result will be 0xA1A1, if any test is FAILED the corresponding result   } 
{ will be 0xDEAD (or at least not 0xA1A1).                                     } 
{------------------------------------------------------------------------------} 
 
test_setup:	AR  = 0xDEAD; 
		DM(jump_test_result) = AR; 
		DM(call_test_result) = AR; 
		DM(loop_test_result) = AR; 
		DM(dp_test_result) = AR; 
		DM(ALU_func_result) = AR; 
		DM(DAG1_result) = AR; 
		DM(DAG2_result) = AR; 
		DM(MAC_result) = AR; 
		DM(SHIFTER_result) = AR; 
		DM(TIMER_result) = AR; 
 
{------------------------------------------------------------------------------} 
{ This section of the code tests some program flow instructions                } 
{ Tested: jump, conditional jump, call, conditional call, rts                  } 
{------------------------------------------------------------------------------} 
 
flow_test:	AR  = 0xA1A1; 
		DM(jump_test_result) = AR; 
		AR  = 0xDEAD; 
		JUMP flowtest1_pass; 
 
		DM(jump_test_result) = AR;	{ test jump } 
flowtest1_pass:	AF  = PASS 1; 
		IF GT JUMP flowtest2_pass; 
 
		DM(jump_test_result) = AR;	{ test conditional jump } 
flowtest2_pass:	CALL flowtest3; 
		IF NE CALL flowtest4; 
		JUMP loop_test;			{ test is finished } 
		IDLE; 
 
flowtest3:	AR  = 0xA100;			{ test call function } 
		DM(call_test_result) = AR; 
		RTS; 
 
flowtest4:	AR  = 0x00A1;			{ test condition call function } 
		AY0 = DM(call_test_result); 
		AR  = AR OR AY0; 
		DM(call_test_result) = AR; 
		RTS; 
 
{------------------------------------------------------------------------------} 
{ This section tests zero-overhead looping in the Program Sequencer            } 
{ - Coverage of: counter, count stack, loop stack, loop comparator             } 
{ - Partial Coverage of: PC stack, condition logic, next address mux           } 
{------------------------------------------------------------------------------} 
 
loop_test:	AY0 = 0; 
		AY1 = 0;		{ AY0, AY1 used to hold count } 
		AF  = PASS 1;		{ AF used as increment value } 
		MR  = 0;		{ MR1,MR0 used to hold count } 
		SR0 = 0;		{ SR0 used to hold count } 
		CNTR = 2;		{ set counter for outer loop } 
		DO testloop1 UNTIL CE; 
		   AR  = AY0 + 1; 
		   AY0 = AR; 
		   CNTR = 3; 
		   DO testloop2 UNTIL CE; 
		      AR  = AY1 + 1; 
		      AY1 = AR; 
		      CNTR = 2; 
		      DO testloop3 UNTIL CE; 
		         AR  = MR0 + AF; 
		         MR0 = AR; 
		         CNTR = 3; 
		         DO testloop4 UNTIL CE; 
		            AR  = MR1 + AF; 
		            MR1 = AR; 
		            CNTR = 2; 
testloop5:	               AR  = SR0 + AF; 
		               SR0 = AR; 
		               IF NOT CE JUMP testloop5; 
testloop4:	            NOP; 
testloop3:	         NOP; 
testloop2:	      NOP; 
testloop1:	   NOP; 
 
		AX1 = 2; 
		AR  = AX1 - AY0;		{ check testloop1 count } 
		IF NE JUMP loop_test_fail; 
 
		AX1 = 6; 
		AR  = AX1 - AY1;		{ check testloop2 count } 
		IF NE JUMP loop_test_fail; 
 
		AY0 = 12; 
		AR  = MR0 - AY0;		{ check testloop3 count } 
		IF NE JUMP loop_test_fail; 
 
		AY0 = 36; 
		AR  = MR1 - AY0;		{ check testloop4 count } 
		IF NE JUMP loop_test_fail; 
 
		AY0 = 72; 
		AR  = SR0 - AY0;		{ check testloop5 count } 
		IF NE JUMP loop_test_fail; 
 
loop_test_pass:	AR  = 0xA1A1; 
		DM(loop_test_result) = AR; 
 
loop_test_fail:	NOP; 
 
 
{------------------------------------------------------------------------------} 
{ This section tests the datapaths through the ALU, MAC and Shifter            } 
{ - Tests both the primary and secondary register sets                         } 
{ - Does NOT check: AF, MR2, MF, SE, SB                                        } 
{------------------------------------------------------------------------------} 
 
datapaths:	CNTR = 2; 
		DO dp_loop1 UNTIL CE; 
		   AF  = PASS 0; 
		   AY0 = 0xA5A5;		{ test datapaths using 0xA5A5 } 
		   CNTR = 2; 
		   DO dp_loop2 UNTIL CE; 
		      AY1 = AY0; 
		      AX0 = AY1; 
		      AX1 = AX0; 
		      AR  = AX1; 
		      MX0 = AR; 
		      MY0 = MX0; 
		      MX1 = MY0; 
		      MY1 = MX1; 
		      MR0 = MY1; 
		      MR1 = MR0; 
		      SI  = MR1; 
		      SR0 = SI; 
		      SR1 = SR0; 
 
		      AR  = SR1 - AY0; 
		      IF NE AF = PASS 1; 
dp_loop2:	      AY0 = 0x5A5A;		{ test datapaths using 0x5A5A } 
dp_loop1:	   ENA SEC_REG;			{ test secondary registers } 
 
		AR = PASS AF; 
		IF NE JUMP datapaths_fail; 
		DIS SEC_REG; 
		AR  = PASS AF; 
		IF NE JUMP datapaths_fail; 
 
datapaths_pass:	AR  = 0xA1A1; 
		DM(dp_test_result) = AR; 
 
datapaths_fail:	DIS SEC_REG; 
 
{------------------------------------------------------------------------------} 
{ This section tests the Arithmetic Logic Unit                                 } 
{------------------------------------------------------------------------------} 
 
ALU_func:	AY0 = 0x5A5A;			{ test the AF register } 
		AF  = PASS AY0; 
		AR  = PASS AF; 
		AR  = AR - AY0; 
		IF NE JUMP ALU_func_fail; 
 
		AX0 = 0xA5A5;			{ test several ALU functions } 
		AF  = PASS AX0; 
		AF  = AX0 + AF; 
		AF  = AX0 - AF; 
		AF  = AX0 XOR AF; 
		AF  = AX0 AND AF; 
		AF  = - AF; 
		AR  = AF - 1; 
		AY0 = 0x5A5B; 
		AR  = AR - AY0; 
		IF NE JUMP ALU_func_fail; 
 
ALU_func_pass:	AR  = 0xA1A1; 
		DM(ALU_func_result) = AR; 
 
ALU_func_fail:  NOP; 
 
{------------------------------------------------------------------------------} 
{ This section tests Data Address Generator 1                                  } 
{ - First the datapaths are tested within DAG1                                 } 
{ - Second, the post modify and circular buffering of DAG1 is tested           } 
{------------------------------------------------------------------------------} 
 
{ Test DAG1 datapaths } 
 
DAG1_datapaths:	AF  = PASS 0; 
		AX0 = 0x1A5A; 
		CNTR= 2; 
		DO DAG1_dptest UNTIL CE; 
		  I0 = AX0; 
		  M0 = I0; 
		  L0 = M0; 
		  I1 = L0; 
		  M1 = I1; 
		  L1 = M1; 
		  I2 = L1; 
		  M2 = I2; 
		  L2 = M2; 
		  I3 = L2; 
		  M3 = I3; 
		  L3 = M3; 
 
		  AY0 = L3; 
 		  AR  = AX0 - AY0; 
		  IF NE AF = PASS 1; 
 
DAG1_dptest:	  AX0 = 0x25A5; 
 
		AR  = PASS AF; 
		IF NE JUMP DAG1_test_fail; 
 
{ test post-modify and circular buffering } 
 
DAG1_func:	I2  = 0x3800; 
		L2  = 18; 
		M0  = 14; 
		M1  = -7; 
		M2  = 15; 
		M3  = -17; 
 
		AR  = DM(I2,M0);	{ test post-addition } 
		MODIFY(I2,M1);		{ test post-subtraction } 
		AR  = DM(I2,M2);	{ test positive circular wrap-around } 
		MODIFY(I2,M3);		{ test negative circular wrap-around } 
 
		AX0 = I2; 
		AY0 = 0x3805; 
		AR  = AX0 - AY0; 
		IF NE JUMP DAG1_test_fail; 
 
DAG1_test_pass:	AR  = 0xA1A1; 
		DM(DAG1_result) = AR; 
 
DAG1_test_fail:	NOP; 
 
{------------------------------------------------------------------------------} 
{ This section tests Data Address Generator 2                                  } 
{ - First the datapaths are tested within DAG1                                 } 
{ - Second, the post modify and circular buffering of DAG1 is tested           } 
{------------------------------------------------------------------------------} 
 
{ Test DAG2 datapaths } 
 
DAG2_datapaths:	AF  = PASS 0; 
		AX0 = 0x1A5A; 
		CNTR= 2; 
		DO DAG2_dptest UNTIL CE; 
		  I4 = AX0; 
		  M4 = I4; 
		  L4 = M4; 
		  I5 = L4; 
		  M5 = I5; 
		  L5 = M5; 
		  I6 = L5; 
		  M6 = I6; 
		  L6 = M6; 
		  I7 = L6; 
		  M7 = I7; 
		  L7 = M7; 
 
		  AY0 = L7; 
 		  AR  = AX0 - AY0; 
		  IF NE AF = PASS 1; 
 
DAG2_dptest:	  AX0 = 0x25A5; 
 
		AR  = PASS AF; 
		IF NE JUMP DAG2_test_fail; 
 
{ test post-modify and circular buffering } 
 
DAG2_func:	I6  = 0x3800; 
		L6  = 18; 
		M4  = 14; 
		M5  = -7; 
		M6  = 15; 
		M7  = -17; 
 
		AR  = DM(I6,M4);	{ test post-addition } 
		MODIFY(I6,M5);		{ test post-subtraction } 
		AR  = DM(I6,M6);	{ test positive circular wrap-around } 
		MODIFY(I6,M7);		{ test negative circular wrap-around } 
 
		AX0 = I6; 
		AY0 = 0x3805; 
		AR  = AX0 - AY0; 
		IF NE JUMP DAG2_test_fail; 
 
DAG2_test_pass:	AR  = 0xA1A1; 
		DM(DAG2_result) = AR; 
 
DAG2_test_fail:	NOP; 
 
{------------------------------------------------------------------------------} 
{ This section tests the Multiplier/Accumulator                                } 
{------------------------------------------------------------------------------} 
 
MAC_func:	MY0 = 0xA5A5; 
		MX0 = 1; 
		MR  = 0; 
		MR  = MR + MX0 * MY0 (SS); 
		MR  = MR + MR0 * MY0 (SS); 
		MR  = MR + MR1 * MY0 (SS); 
		MR  = MR - MR2 * MY0 (SS); 
 
		AX0 = MR0; 
		AY0 = 0x3DB2; 
		AR  = AX0 - AY0; 
		IF NE JUMP MAC_func_fail; 
 
		AX0 = MR1; 
		AY0 = 0xF05E; 
		AR  = AX0 - AY0; 
		IF NE JUMP MAC_func_fail; 
 
MAC_func_pass:	AR  = 0xA1A1; 
		DM(MAC_result) = AR; 
 
MAC_func_fail:	NOP; 
 
{------------------------------------------------------------------------------} 
{ This section tests the Barrel Shifter                                        } 
{------------------------------------------------------------------------------} 
 
SHIFTER_func:	SI  = 0x1234; 
		SR  = ASHIFT SI BY 8 (HI); 
		SI  = 0x5678; 
		SR  = SR OR ASHIFT SI BY 8 (LO); 
		SE  = EXP SR1 (HI); 
 
		AY0 = 0x3456; 
		AR  = SR1 - AY0; 
		IF NE JUMP SHIFTER_fail; 
 
		AY0 = -1; 
		AX0 = SE; 
		AR  = AX0 - AY0; 
		IF NE JUMP SHIFTER_fail; 
 
SHIFTER_pass:	AR  = 0xA1A1; 
		DM(SHIFTER_result) = AR; 
 
SHIFTER_fail:	NOP; 
 
{------------------------------------------------------------------------------} 
{ This section tests the TIMER and the associated interrupt.                   } 
{------------------------------------------------------------------------------} 
 
TIMER_test:	AX0 = 1; 
		DM(0x3FFB) = AX0;	{ load TSCALE } 
		AX0 = 5; 
		DM(0x3FFC) = AX0;	{ load TCOUNT } 
		DM(0x3FFD) = AX0;	{ load TPERIOD } 
		AX0  = 0xDEAD; 
		IMASK = 0x01; 
		ENA TIMER;		{ timer interrupt in 12 cycles } 
 
		CNTR = 11; 
		DO timer_wait UNTIL CE; 
timer_wait:	   NOP; 
		IMASK = 0; 
		DIS TIMER; 
		 
TIMER_done:	DM(TIMER_result) = AX0; 
 
{------------------------------------------------------------------------------} 
{ This completes the tests of the internal sections of the ADSP-2101.  After   } 
{ successful completion, the first ten locations of data memory will be        } 
{ 0xA1A1.                                                                      } 
{------------------------------------------------------------------------------} 
 
results_test:	I0  = ^jump_test_result;	{ points to test_results } 
		L0  = 0; 
		M1  = 1; 
		AY0 = 0xA1A1; 
		AF  = PASS 0; 
 
		CNTR = 10; 
		DO results_loop UNTIL CE; 
		   AX0 = DM(I0,M1); 
		   AR  = AX0 - AY0; 
results_loop:	   IF NE AF = PASS 1; 
 
		AR  = PASS AF; 
		IF NE JUMP full_test_fail; 
		JUMP DMTEST; 
 
full_test_fail:	IDLE; 
 
{------------------------------------------------------------------------------} 
{ This section tests internal data memory RAM. It uses DAG1 for addressing.    } 
{------------------------------------------------------------------------------} 
 
DMTEST:		nop; 
 
.ENDMOD; 
